import IKGlobals from './../IKGlobals.js';
import Utils from './../API/IKUtils.js';
import IKResponse, { IKBadRequestResponse } from './../API/IKResponse.js';
//https://www.npmjs.com/package/superstruct
import * as struct from 'superstruct';
//validation libs
import isUuid from 'is-uuid';
import isEmail from 'is-email';
//
export default class IKValidation {
  constructor(config) {
    this.customStruct = {};
    this._buildInternalTypes((config && config.additionalTypes ? config.additionalTypes : {}));
  }
  _buildInternalTypes(additionalTypes) {
    for (const key in IKValidation.internalTypes()) {
      this.customStruct[key] = struct.define(key, IKValidation.internalTypes()[key]);
    }
    for (const key in additionalTypes) {
      this.customStruct[key] = struct.define(key, additionalTypes[key]);
    }
  }
  async validate(spec, obj, request) {
    let assertion = null;
    let err = null;
    try {
      let parsedRawObj = obj;
      if (typeof spec == 'function') spec = spec(this)
      assertion = struct.assert(parsedRawObj, spec);
    } catch (e) {
      console.debug(e);
      const { path, value, type } = e;
      const key = (path ? path[0] : undefined);
      let errorMessage = e;
      //Check for value required error
      if (value === undefined) errorMessage = `Required value '${key}' is missing.`
      else if (type === undefined) errorMessage = `Unknown key '${key}'.`
      else errorMessage = `Invalid value '${value}' for key '${key}'.`
      //Generate error
      err = IKBadRequestResponse(IKGlobals.ErrorResponseValidationFail + errorMessage, IKGlobals.ErrorCode_InvalidInput);
    }
    //Catch error as response
    if (err) return err;
    //run normal code out of this try/catch scope
    return await request(obj);
  }
}
//
IKValidation.internalTypes = function() {
  return {
    email: value => { return (isEmail(value) && value.length < 256); },
    fullAddress: value => {
      if (!value) return false;
      return true;
    },
    'date-yyyymmdd': value => {
      if (!value) return false;
      //TODO CHECK DATE FORMAT
      let d = new Date(value);
      return (d instanceof Date && !isNaN(d));
    },
    'arrayIDs': value => {
      if (!value || !Array.isArray(value)) return false;
      for (let id of value) if (id.length < 10) return false;
      return true;
    },
    //TODO: this should be moved out to the project requiring this
    ACEArray: value => {
      if (!value || !value.length) return false;
      for (let val of value) {
        if (!val.accessType || !val.roleID) return false;
      }
      return true;
    },
  }
}